@@ -63,7 +63,6 @@ def execute_order(
         datetime_datetime: datetime,
         string_crypto_key: str,
         float_number_to_be_bought: float,
-        int_strategy_id: int,
         df_prices: DataFrame,
         df_trading_journal: DataFrame,
         float_margin_loan_rate: float,
@@ -74,7 +73,7 @@ def execute_order(
         dict_constraints: dict,
         dict_general_settings: dict,
         boolean_allow_partially_filled_orders: bool
-    ):
+    ) -> dict:
     """Executes all kinds of dict_orders.
 
     Can handle more than one dict_order per point in time by subsequently calling
@@ -83,9 +82,11 @@ def execute_order(
     if boolean_buy and (float_number_to_be_bought < 0) or not boolean_buy and (float_number_to_be_bought > 0):
         raise ValueError(f'boolean_buy: {boolean_buy} and float_number_to_be_bought: {float_number_to_be_bought} is contradictory.')
 
+    if boolean_buy == None or float_number_to_be_bought == 0:
+        raise ValueError(f'boolean_buy: {boolean_buy} or float_number_to_be_bought: {float_number_to_be_bought} cannot lead to an order.')
+
     dict_order = {
         'datetime': datetime_datetime,
-        'Strategy ID': int_strategy_id,
         'Asset': string_crypto_key,
         'Buy or sell': boolean_buy
     }
@@ -187,7 +188,7 @@ def execute_order(
             return float_number_to_be_bought
         else:
             if float_initial_number_to_be_bought != float_number_to_be_bought:
-                raise InputError('Quantity {float_number_to_be_bought} for {string_crypto_key} cannot be covered with the given budget or dict_constraints. Maximum {max_possible_quantity} units can be bought.')
+                raise ValueError(f'Quantity {float_initial_number_to_be_bought} for {string_crypto_key} cannot be covered with the given budget or dict_constraints. Maximum {float_number_to_be_bought} units can be bought.')
             else:
                 return float_number_to_be_bought
 
@@ -240,11 +241,11 @@ def execute_order(
                 if dict_display_options['boolean_warning_buy_order_could_not_be_filled']:
                     print(f'Order execution warning: Buy dict_order for {float_number_to_be_bought} units of {string_crypto_key} could not be filled.')
             elif float_number_to_be_bought < float_initial_number_to_be_bought:
-                raise ValueError('It is not possible that the signal quantity {float_initial_number_to_be_bought} is higher (less assets sold) than the final quantity {float_number_to_be_bought} for {order["Asset"]}.')
+                raise ValueError(f'It is not possible that the signal quantity {float_initial_number_to_be_bought} is higher (less assets sold) than the final quantity {float_number_to_be_bought} for {order["Asset"]}.')
             return float_number_to_be_bought
         else:
             if float_number_to_be_bought != float_initial_number_to_be_bought:
-                raise InputError('Quantity {float_number_to_be_bought} for {string_crypto_key} cannot be sold with the given assets or dict_constraints. Maximum {float_number_to_be_bought} units can be sold.')
+                raise ValueError(f'Quantity {float_number_to_be_bought} for {string_crypto_key} cannot be sold with the given assets or dict_constraints. Maximum {float_number_to_be_bought} units can be sold.')
             else:
                 return float_number_to_be_bought
 
@@ -276,6 +277,18 @@ def execute_order(
         float_number_to_be_bought = 0
         float_price = 0
 
+    if float_price != find_price(
+        df_prices=df_prices,
+        desired_index=(datetime_datetime, string_crypto_key),
+        boolean_allow_older_prices=False,
+        boolean_allow_newer_prices=False,
+        boolean_warnings=False,
+        boolean_errors=False,
+        timedelta_to_allow_to_go_back=Timedelta(days=0),
+        timedelta_to_allow_to_go_to_the_future=Timedelta(days=0)
+    ):
+        raise ValueError(f'The indicated price {float_price} from the strategy is wrong. Date: {datetime_datetime}')
+
     dict_order['Dict of assets in portfolio'][string_crypto_key] = dict_order['Dict of assets in portfolio'][string_crypto_key] + float_number_to_be_bought
 
     if float_number_to_be_bought != 0:
@@ -580,12 +593,17 @@ def prepare_signal_list_san(
     except:
         raise ValueError("Please set up a signal table. A signal table needs the following columns: 'date' (yyyy-mm-dd), 'signal_strength' (numeric that is used to infer buy or sell dict_orders), 'string_id' (hexadecimal ERC20 token identifier or ITIN). You can change the column names in the Excel/CSV file to fit this convention or in main.py so that the program adjusts to the naming in the table.")
 
-    df_signals['signal_type'] = where(df_signals['signal_strength']<=dict_strategy_hyperparameters['sell_parameter'],'SELL',where(df_signals['signal_strength']>=dict_strategy_hyperparameters['buy_parameter'],'BUY','HODL'))
-    df_signals = df_signals[~df_signals['signal_type'].isin(['HODL'])]
+    print(df_signals)
+
+    try:
+        test = df_signals['signal_type']
+    except:
+        df_signals['signal_type'] = where(df_signals['signal_strength']<=dict_strategy_hyperparameters['sell_parameter'],'SELL',where(df_signals['signal_strength']>=dict_strategy_hyperparameters['buy_parameter'],'BUY','HODL'))
+        df_signals = df_signals[~df_signals['signal_type'].isin(['HODL'])]
 
     return df_signals
 
-def execute_strategy_multi_asset(
+def execute_strategy_signal_table(
         df_prices,
         int_chosen_strategy,
         float_budget_in_usd,
@@ -635,9 +653,9 @@ def execute_strategy_multi_asset(
         )
 
         if float_price is not None and float_price > 0:
-            if dict_strategy_hyperparameters['boolean_percentage_in_signal_table']:
+            if dict_strategy_hyperparameters['boolean_percentage_in_relation_to_total_portfolio_value_in_signal_table']:
                 if len(df_trading_journal) > 0:
-                    percentage_that_is_still_missing = row['percentage_to_be_bought'] - (
+                    percentage_that_is_still_missing = row['percentage_to_be_bought_in_relation_to_total_portfolio'] - (
                         (float_price * df_trading_journal['Dict of assets in portfolio'].iloc[-1][string_crypto_key]) / calculate_portfolio_value(
                             df_prices=df_prices,
                             dict_of_assets_in_portfolio=df_trading_journal['Dict of assets in portfolio'].iloc[-1],
@@ -656,7 +674,7 @@ def execute_strategy_multi_asset(
                         dict_general_settings['int_rounding_decimal_places_for_security_quantities']
                     )
                 else:
-                    percentage_that_is_still_missing = row['percentage_to_be_bought']
+                    percentage_that_is_still_missing = row['percentage_to_be_bought_in_relation_to_total_portfolio']
                     float_number_to_be_bought = round(
                         (
                             (float_budget_in_usd * percentage_that_is_still_missing) - dict_constraints['float_minimum_cash']
@@ -670,6 +688,60 @@ def execute_strategy_multi_asset(
                 else:
                     boolean_buy = None
 
+            elif dict_strategy_hyperparameters['boolean_iterative_percentage_approach_signal_table']:
+                if row['signal_type'] == 'BUY':
+                    if len(df_trading_journal) > 0:
+                        float_number_to_be_bought = round(
+                            (
+                                (df_trading_journal['Cash'].iloc[-1] * row['percentage_to_be_bought_in_relation_to_cash']) - dict_constraints['float_minimum_cash']
+                            ) / float_price,
+                            dict_general_settings['int_rounding_decimal_places_for_security_quantities']
+                        )
+                    else:
+                        print(row)
+                        float_number_to_be_bought = round(
+                            (
+                                (float_budget_in_usd * row['percentage_to_be_bought_in_relation_to_cash']) - dict_constraints['float_minimum_cash']
+                            ) / float_price,
+                            dict_general_settings['int_rounding_decimal_places_for_security_quantities']
+                        )
+                    if float_number_to_be_bought > 0:
+                        boolean_buy = True
+                    elif float_number_to_be_bought < 0:
+                        raise ValueError(f'float_number_to_be_bought cannot be negative ({float_number_to_be_bought}) here.')
+                    else:
+                        boolean_buy = None
+
+                elif row['signal_type'] == 'SELL':
+                    if len(df_trading_journal) > 0:
+                        float_number_to_be_bought = (-1) * round(
+                            (
+                                calculate_portfolio_value(
+                                    df_prices=df_prices,
+                                    dict_of_assets_in_portfolio=df_trading_journal['Dict of assets in portfolio'].iloc[-1],
+                                    datetime_datetime=index[0],
+                                    dict_constraints=dict_constraints,
+                                    dict_display_options=dict_display_options,
+                                    int_rounding_accuracy=dict_general_settings['int_rounding_decimal_places'],
+                                    float_cash_value=0,  # Only the asset value is relevant here.
+                                    boolean_gross_exposure=False
+                            ) * row['percentage_to_be_sold_in_relation_to_asset']
+                        ) / float_price,
+                            dict_general_settings['int_rounding_decimal_places_for_security_quantities']
+                        )
+                    else:
+                        float_number_to_be_bought = 0  # No sell possible if no trades were recorded yet.
+
+                    if float_number_to_be_bought > 0:
+                        raise ValueError(f'float_number_to_be_bought cannot be positive ({float_number_to_be_bought}) here.')
+                    elif float_number_to_be_bought < 0:
+                        boolean_buy = False
+                    else:
+                        boolean_buy = None
+
+                else:
+                    raise ValueError(f"The signal_type column should only contain BUY or SELL signals, not {row['signal_type']}")
+
             elif row['signal_type'] == "BUY":
                 boolean_buy = True
 
@@ -729,7 +801,7 @@ def execute_strategy_multi_asset(
                     float_number_to_be_bought = None
 
             else:
-                raise ValueError('Ambiguous signal:', row['signal_type'])
+                raise ValueError(f"Ambiguous signal: {row['signal_type']}")
 
             try:
                 amount = df_trading_journal['Cash'].iloc[-1]
@@ -741,33 +813,35 @@ def execute_strategy_multi_asset(
 
             float_number_to_be_bought = round(float_number_to_be_bought, dict_general_settings['int_rounding_decimal_places_for_security_quantities'])
 
-            dataseries_trading_journal = execute_order(
-                boolean_buy=boolean_buy,
-                datetime_datetime=index[0],
-                int_strategy_id=int_chosen_strategy,
-                string_crypto_key=string_crypto_key,
-                float_number_to_be_bought=float_number_to_be_bought,
-                df_prices=df_prices,
-                df_trading_journal=df_trading_journal,
-                float_margin_loan_rate=float_margin_loan_rate,
-                float_budget_in_usd=float_budget_in_usd,
-                float_price=float_price,
-                dict_fees={
-                    'float_absolute_fee_buy_order':dict_crypto_options['dict_general']['float_absolute_fee_buy_order'],
-                    'float_absolute_fee_sell_order':dict_crypto_options['dict_general']['float_absolute_fee_sell_order'],
-                    'float_percentage_buying_fees_and_spread':dict_crypto_options['dict_general']['float_percentage_buying_fees_and_spread'],
-                    'float_percentage_selling_fees_and_spread':dict_crypto_options['dict_general']['float_percentage_selling_fees_and_spread']
-                },
-                dict_display_options=dict_display_options,
-                dict_constraints=dict_constraints,
-                dict_general_settings=dict_general_settings,
-                boolean_allow_partially_filled_orders=dict_strategy_hyperparameters['boolean_allow_partially_filled_orders']
-            )
+            if float_number_to_be_bought != 0 and (boolean_buy or boolean_buy == False):
+                dataseries_trading_journal = execute_order(
+                    boolean_buy=boolean_buy,
+                    datetime_datetime=index[0],
+                    string_crypto_key=string_crypto_key,
+                    float_number_to_be_bought=float_number_to_be_bought,
+                    df_prices=df_prices,
+                    df_trading_journal=df_trading_journal,
+                    float_margin_loan_rate=float_margin_loan_rate,
+                    float_budget_in_usd=float_budget_in_usd,
+                    float_price=float_price,
+                    dict_fees={
+                        'float_absolute_fee_buy_order':dict_crypto_options['dict_general']['float_absolute_fee_buy_order'],
+                        'float_absolute_fee_sell_order':dict_crypto_options['dict_general']['float_absolute_fee_sell_order'],
+                        'float_percentage_buying_fees_and_spread':dict_crypto_options['dict_general']['float_percentage_buying_fees_and_spread'],
+                        'float_percentage_selling_fees_and_spread':dict_crypto_options['dict_general']['float_percentage_selling_fees_and_spread']
+                    },
+                    dict_display_options=dict_display_options,
+                    dict_constraints=dict_constraints,
+                    dict_general_settings=dict_general_settings,
+                    boolean_allow_partially_filled_orders=dict_strategy_hyperparameters['boolean_allow_partially_filled_orders']
+                )
 
-            df_trading_journal = df_trading_journal.append(
-                dataseries_trading_journal,
-                ignore_index=True
-            )
+                dataseries_trading_journal['Strategy ID'] = int_chosen_strategy
+
+                df_trading_journal = df_trading_journal.append(
+                    dataseries_trading_journal,
+                    ignore_index=True
+                )
 
     dict_comments['constraints'] = dict_constraints
     dict_comments['general_settings'] = dict_general_settings
@@ -852,7 +926,6 @@ def execute_strategy_white_noise(
             df_trading_journal=df_trading_journal,
             boolean_buy=boolean_buy,
             datetime_datetime=index[0],
-            int_strategy_id=int_chosen_strategy,
             string_crypto_key=string_crypto_key,
             float_number_to_be_bought=float_number_to_be_bought,
             float_margin_loan_rate=float_margin_loan_rate,
@@ -870,6 +943,8 @@ def execute_strategy_white_noise(
             boolean_allow_partially_filled_orders=dict_strategy_hyperparameters['boolean_allow_partially_filled_orders']
         )
 
+        dataseries_trading_journal['Strategy ID'] = int_chosen_strategy
+
         df_trading_journal = df_trading_journal.append(
             dataseries_trading_journal,
             ignore_index=True
@@ -1000,7 +1075,6 @@ def execute_strategy_ma_crossover(
                 dataseries_trading_journal = execute_order(
                     boolean_buy=boolean_buy,
                     datetime_datetime=datetime_current_time,
-                    int_strategy_id=int_chosen_strategy,
                     string_crypto_key=dict_strategy_hyperparameters['string_id'],
                     float_number_to_be_bought=float_number_to_be_bought,
                     df_prices=df_prices,
@@ -1020,6 +1094,8 @@ def execute_strategy_ma_crossover(
                     boolean_allow_partially_filled_orders=dict_strategy_hyperparameters['boolean_allow_partially_filled_orders']
                 )
 
+                dataseries_trading_journal['Strategy ID'] = int_chosen_strategy
+
                 df_trading_journal = df_trading_journal.append(
                     dataseries_trading_journal,
                     ignore_index=True
@@ -1077,7 +1153,7 @@ def test_strategy(
     elif int_chosen_strategy == 2:
         raise NotImplementedError('Strategy 2 is not implemented.')
     elif int_chosen_strategy == 3:
-        dict_execution_results = execute_strategy_multi_asset(
+        dict_execution_results = execute_strategy_signal_table(
             df_prices=df_prices,
             float_budget_in_usd=float_budget_in_usd,
             int_chosen_strategy=int_chosen_strategy,
