@@ -1,7 +1,9 @@
-__version__ = "0.0.32"
+from pathlib import Path
+
+with open(Path(__file__).parent.parent.joinpath("VERSION")) as f:
+    __version__ = f.readline()
 
 # Importing modules from this repository
-
 from _0_wrappers import backtest_visualizer
 
 
